const readline = require('readline')
const fs = require('fs')
const http = require('http');
const https = require('https');

// Confs
const config = require('./config.json')

const auth = 'Basic ' + Buffer.from(config.gupshup.proxy.user + ':' + config.gupshup.proxy.pass).toString('base64')

const xserver = process.argv[2] ? process.argv[2] : "http://csadev.xcontactcenter.com.br:8000"
const clientID = process.argv[3] ? process.argv[3] : "31434d6c-9358-42ae-89eb-42f0034e8d44"

// Apenas para fins de teste
const rl = readline.createInterface({
    input: process.stdin,
    output: process.stdout,
    prompt: 'CSAv3 CLI> '
});

let client;

http.request({
  host: config.gupshup.proxy.host, // IP address of proxy server
  port: config.gupshup.proxy.port, // port of proxy server
  method: 'CONNECT',
  path: 'csadev.xcontactcenter.com.br:8000', // destination
  headers: {
    'Proxy-Authorization': auth
  },
}).on('connect', (res, socket) => {
  console.log('Proxy StatusCod: ', res.statusCode)
  console.log('Proxy statusMessage:', res.statusMessage)
  if (res.statusCode === 200) { // connected to proxy server
    console.log(`***** Conectou no proxy *****`)
    const agent = new http.Agent({ socket });

    client = require("socket.io-client")(xserver, {
        "rejectUnauthorized": false,
        agent
    });
    client.on('connect', () => {
        console.log('Bem vindo ao CSA v3 client, você está conectado.');
        client.emit(`Login`, { clientID: clientID }, response => {
            console.log(`Retorno de login ${JSON.stringify(response, null, 2)}`);
            rl.prompt();
        })
        rl.prompt();
    });
    
    client.on('message', (msg) => {
        console.log(msg);
        rl.prompt();
    });
    // https.get({
    //   host: 'www.google.com:443',
    //   path: '/',
    //   agent,      // cannot use a default agent
    // }, (res) => {
    //   let chunks = []
    //   res.on('data', chunk => chunks.push(chunk))
    //   res.on('end', () => {
    //     console.log('DONE', Buffer.concat(chunks).toString('utf8'))
    //   })
    // })
  }
}).on('error', (err) => {
  console.error('error', err)
}).end();

// const client = require("socket.io-client")(xserver, {
//     "rejectUnauthorized": false
// });

rl.on('line', (line) => {
    let pars = line.trim().split(' ');

    switch (pars[0]) {
        case 'sair':
        case 'quit':
        case 'q':
            rl.close();
            break;
        case '':
            break;
        case 'login':
            client.emit(`Login`, { clientID: pars[1] }, response => {
                console.log(`Retorno de login ${JSON.stringify(response, null, 2)}`);
                rl.prompt();
            })
            break;
        case 'start':
            client.emit(`startInstance`, { instanceID: pars[1] }, response => {
                console.log(`Retorno de startInstance ${JSON.stringify(response, null, 2)}`);
                rl.prompt();
            })
            break;
        case 'status':
            client.emit(`getWhatsStatus`, { instanceID: pars[1] }, response => {
                console.log(`Retorno de getWhatsStatus ${JSON.stringify(response, null, 2)}`);
                rl.prompt();
            })
            break;
        case 'stop':
            client.emit(`stopInstance`, { instanceID: pars[1] }, response => {
                console.log(`Retorno de stopInstance ${JSON.stringify(response, null, 2)}`);
                rl.prompt();
            })
            break;
        case 'enviar':
            client.emit(`newWhatsMsg`, { instanceID: pars[1], chatId: pars[2], msg: pars[3] }, response => {
                console.log(`Retorno de newWhatsMsg ${JSON.stringify(response, null, 2)}`);
                rl.prompt();
            })
            break;
        case 'responder':
            //client.emit('replyMessage', { instanceID: pars[1], chatId: pars[2], id: pars[3], msg: pars[4], options: { sendMediaAsDocument: true } }, resposta => {
            client.emit('replyMessage', { instanceID: pars[1], chatId: pars[2], id: pars[3], msg: pars[4], options: { fromMe: pars[5] ? true : false } }, resposta => {
                console.log(`replyMessage resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break;
        case 'apagarchat':
            client.emit('deleteChat', { instanceID: pars[1], chatId: pars[2] }, resposta => {
                console.log(`deleteChat resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break
        case 'apagar':
            client.emit('deleteMessage', { instanceID: pars[1], chatId: pars[2], id: pars[3] }, resposta => {
                console.log(`deleteMessage resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break
        case 'ler':
            client.emit('readMessage', { instanceID: pars[1], chatId: pars[2], id: pars[3] }, resposta => {
                console.log(`readMessage resposta: ${JSON.stringify(resposta, null, 2)}`);
                rl.prompt();
            });
            break
        default:
            break;
    }
    rl.prompt();
}).on('close', () => {
    console.log('Até logo!');
    process.exit(0);
});